function [alpha1 beta1 Le12] = turbo_half_decode(ys, e1n, alpha1, beta1, Le21, Lc, S)
% ys = noisy message
% e1n = noisy pairity check bits
% alpha1 = vector of forward probabilities
% beta1 = vector of forward probabilities
% if beta1 is empty, assume we are on decoder stage 2 and set equal to
% alpha(N)
% Le21 = input extrinsic information
% Le12 = output extrinsic information



len_x = length(ys);

% initialize variables
gammaE=zeros(2,len_x,16);
gamma1=zeros(2,len_x,16);
Le12 = zeros(1,len_x);

% calculate gamma, alpha
for kk=1:len_x
    gammaE(1,kk,:) = exp(0.5*Lc*e1n(kk)*S.zero_out_ap);
    gammaE(2,kk,:) = exp(0.5*Lc*e1n(kk)*S.one_out_ap);    
    gamma1(1,kk,:) = exp(0.5*-1*(Le21(kk)+Lc*ys(kk)))*gammaE(1,kk,:);    
    gamma1(2,kk,:) = exp(0.5*1*(Le21(kk)+Lc*ys(kk)))*gammaE(2,kk,:);
    
    g1m = gamma1(1,kk,S.zero_inv(:)+1);
    g1p = gamma1(2,kk,S.one_inv(:)+1);
    denom = sum(alpha1(kk,S.zero_inv(:)+1).*g1m(1,:)+alpha1(kk,S.one_inv(:)+1).*g1p(1,:));

    g1m = gamma1(1,kk,S.zero_inv+1);
    g1p = gamma1(2,kk,S.one_inv+1);
    alpha1(kk+1,:) = (alpha1(kk,S.zero_inv+1).*g1m(1,:)+alpha1(kk,S.one_inv+1).*g1p(1,:))/denom;
end
% calculate beta
if isempty(beta1)
    beta1=zeros(len_x,16);
    beta1(len_x,:)=alpha1(len_x+1,:);   %len_x+1 or len_x?
end
for kk=len_x:-1:2
    % denominator calculation is same as in alpha
    g1m = gamma1(1,kk,S.zero_inv(:)+1);
    g1p = gamma1(2,kk,S.one_inv(:)+1);
    denom = sum(alpha1(kk,S.zero_inv(:)+1).*g1m(1,:)+alpha1(kk,S.one_inv(:)+1).*g1p(1,:));
    % calculate beta using denominator found above
    x=gamma1(1,kk,:);
    y=gamma1(2,kk,:);
    beta1(kk-1,:) = (beta1(kk,S.zero(:)+1).*x(1,:)+beta1(kk,S.one(:)+1).*y(1,:))/denom;    
end
% calculate Le12
for kk=1:len_x
    g1em = gammaE(1,kk,:);
    g1ep = gammaE(2,kk,:);
    num=sum(alpha1(kk,:).*g1ep(1,:).*beta1(kk,S.one(:)+1));
    denom=sum(alpha1(kk,:).*g1em(1,:).*beta1(kk,S.zero(:)+1));
    Le12(kk) = log(num/denom);
end
